@@ -44,6 +44,10 @@ class BasePlugin(object):
     def get_parser(self):
         parser = argparse.ArgumentParser(
             '--plugin', description='A plugin for covid19-outbreak-simulator')
+        parser.add_argument('--disable',
+            action='store_true',
+            help='Disable the plugin if this parameter is specified.'
+        )
         parser.add_argument(
             '--start',
             type=float,
@@ -55,7 +59,7 @@ class BasePlugin(object):
             help='''End time, default to none, meaning there is no end time.''')
         parser.add_argument(
             '--at',
-            nargs='+',
+            nargs='*',
             type=float,
             help='''Specific time at which the plugin is applied.''')
         parser.add_argument(
@@ -71,6 +75,13 @@ class BasePlugin(object):
 
     def get_plugin_events(self, args):
         events = []
+
+        if args.disable:
+            raise RuntimeError(f'Disabled plugin should not be triggered')
+
+        if args.interval is not None and args.interval <= 0:
+            return events
+
         if args.start is not None:
             events.append(
                 PlugInEvent(
